# Generated by Django 4.1.2 on 2022-11-10 18:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('book_manager', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='bookauthor',
            options={'ordering': ('book', 'order'), 'verbose_name': 'book author', 'verbose_name_plural': 'book authors'},
        ),
    ]
